==> legend_pygeom_optics-0.8.0.tar.gz: legend_pygeom_optics-0.8.0/docs/source/exts/optics_plot_extension.py <==
from __future__ import annotations

import ast
import inspect
from pathlib import Path
from typing import Any, Callable

import matplotlib.pyplot as plt
import numpy as np
import pint
from sphinx.application import Sphinx

u = pint.get_application_registry()


def do_plot(
    obj: Callable, plots_dir: Path, safe_name: str, options: dict[str, Any]
) -> list[str]:
    """Create a plot from the given optical property function.

    By default, it will call ``obj()`` and unpack the result into an x-vector, and multiple
    y-vectors. All y-vectors will pe plotted together into one output file.

    Other Parameters
    ----------------
    options
        Change the behaviour of the plot.

        xlim
            Set the plot's x axis limits, see :py:func:`matplotlib.pyplot.xlim`
        ylim
            Set the plot's y axis limits, see :py:func:`matplotlib.pyplot.ylim`
        xscale
            Set the plot's x axis scaling, see :py:func:`matplotlib.pyplot.xscale`
        yscale
            Set the plot's y axis scaling, see :py:func:`matplotlib.pyplot.yscale`
        labels
            Tuple of labels that will be applied if more than one y vector is returned.
        call_x
            Differing from the default behavior above, the function will be called with an
            x vector of wavelengths in the optical range (as :py:class:`pint.Quantity`).
            All return values are interpreted as y vectors.
        standalone
            if True, do not output the return value preamble (i.e. to embed more than one plot)
        ret_offset
            use the argument numbered by this (default: 0) as the first argument that will
            be treated as an x or y vector.
    """
    # init plot
    fig = plt.figure(figsize=(4, 2))
    ax = plt.gca()
    plt.grid()

    ret_offset = options.get("ret_offset", 0)
    if "call_x" in options:
        # special case for LAr properties
        lim = [112 * u.nm, 650 * u.nm]
        if "xlim" in options:
            lim = [xl * u.nm for xl in options["xlim"]]
        x = np.linspace(*lim, num=200)
        ys = obj(x)
        ys = ys[ret_offset:]
        # wrap the result in a tuple, if needed
        ys = ys if isinstance(ys, tuple) else (ys,)
    else:
        data = obj()
        x = data[ret_offset]
        # ys holds one or more
        ys = data[ret_offset + 1 :]

    if isinstance(x, pint.Quantity):
        ax.set_xlabel(x.u)
        x = x.magnitude

    # plot all supplied data vectors
    for i, val in enumerate(ys):
        if isinstance(val, pint.Quantity):
            ax.set_ylabel(val.u)
            y = val.magnitude
        elif isinstance(val, (np.ndarray, list)):
            y = val
        else:
            msg = f"unsupported y-vector type {type(val)} for plot {safe_name}"
            raise ValueError(msg)

        plotoptions = {}
        if "labels" in options:
            plotoptions["label"] = options["labels"][i]

        plt.plot(x, y, marker=".", markersize=2, linewidth=0.5, **plotoptions)

    if len(ys) > 1 and "labels" in options:
        plt.legend()

    # adjust plotting options
    if "ylim" in options:
        ax.set_ylim(options["ylim"])
    if "xlim" in options:
        ax.set_xlim(options["xlim"])
    if "yscale" in options:
        ax.set_yscale(options["yscale"])
    if "xscale" in options:
        ax.set_xscale(options["xscale"])

    # export figure to the filesystem
    fig.tight_layout(pad=0.3)
    fig.savefig(plots_dir / (safe_name + ".png"), dpi=300)

    plt.close()

    lines = []
    if not options.get("standalone", False):
        lines += [":returns:"]
    return [
        *lines,
        f"    .. image:: plots/{safe_name}.png",
        "        :width: 400px",
    ]


def do_const(obj: Callable) -> list[str]:
    """Output the constant return value of the function.

    By default, it will call ``obj()`` and display the numerical value of the return value.
    """
    const = obj()
    description = None

    if isinstance(const, pint.Quantity):
        description = f":math:`{const.m}\\ {const.u:L~}`"
    elif isinstance(const, float):
        description = f":math:`{const}`"
    else:
        msg = ""
        raise ValueError(msg)

    if description is None:
        return [""]

    return [f":returns: constant value {description}"]


def process_docstring(
    app: Sphinx,
    what: str,
    name: str,
    obj: Any,
    options: Any,
    lines: list[str],
) -> None:
    if inspect.isclass(obj) or what != "function":
        return
    if not callable(obj):
        return

    plots_dir = Path(app.srcdir) / "api" / "plots"

    # this only appears to be a sphinx directive, but the parsing here is very simple.
    plot_token = ".. optics-plot::"
    const_token = ".. optics-const::"

    i = 0
    plot_idx = 0
    orig_lines = lines.copy()
    for line in orig_lines:
        if line.startswith(plot_token):
            plots_dir.mkdir(exist_ok=True, parents=True)
            safe_name = "".join(c for c in name if c.isalnum() or c in (".", "-", "_"))
            safe_name += f"_{plot_idx}"

            opt_string = line[len(plot_token) :]
            opts = {}
            if opt_string != "":
                opts = ast.literal_eval(opt_string)
            assert isinstance(opts, dict)

            # replace the custom 'directive' with an actually supported reST directive.
            replace = do_plot(obj, plots_dir, safe_name, opts)
            lines[i : i + 1] = replace
            i += len(replace)
            plot_idx += 1
        elif line.startswith(const_token):
            # replace the custom 'directive' with actually supported reST content.
            replace = do_const(obj)
            lines[i : i + 1] = replace
            i += len(replace)
        else:
            i += 1


def setup(app: Sphinx) -> dict[str, bool]:
    """Register this sphinx extension."""
    app.connect("autodoc-process-docstring", process_docstring)
    return {"parallel_read_safe": True}

==> legend_pygeom_optics-0.9.0.tar.gz: legend_pygeom_optics-0.9.0/docs/source/exts/optics_plot_extension.py <==
from __future__ import annotations

import ast
import inspect
from pathlib import Path
from typing import Any, Callable

import pint
from sphinx.application import Sphinx

from legendoptics.plot import plot_callable

u = pint.get_application_registry()


def do_plot(
    obj: Callable, plots_dir: Path, safe_name: str, options: dict[str, Any]
) -> list[str]:
    """Create a plot from the given optical property function.

    By default, it will call ``obj()`` and unpack the result into an x-vector, and multiple
    y-vectors. All y-vectors will pe plotted together into one output file.

    Other Parameters
    ----------------
    options
        Change the behaviour of the plot. see :meth:`plot_callable`

        standalone
            if True, do not output the return value preamble (i.e. to embed more than one plot)
    """
    file = plots_dir / (safe_name + ".png")

    plot_callable(obj, file, options)

    lines = []
    if not options.get("standalone", False):
        lines += [":returns:"]
    return [
        *lines,
        f"    .. image:: plots/{safe_name}.png",
        "        :width: 400px",
    ]


def do_const(obj: Callable) -> list[str]:
    """Output the constant return value of the function.

    By default, it will call ``obj()`` and display the numerical value of the return value.
    """
    const = obj()
    description = None

    if isinstance(const, pint.Quantity):
        description = f":math:`{const.m}\\ {const.u:L~}`"
    elif isinstance(const, float):
        description = f":math:`{const}`"
    elif hasattr(const, "__to_optics_const__"):
        description = const.__to_optics_const__()
    else:
        msg = f"unsupported type {type(const)}"
        raise RuntimeError(msg)

    if description is None:
        return [""]

    if not isinstance(description, str):
        return [f":returns: constant value {description[0]}", *description[1:]]
    return [f":returns: constant value {description}"]


def process_docstring(
    app: Sphinx,
    what: str,
    name: str,
    obj: Any,
    options: Any,
    lines: list[str],
) -> None:
    if inspect.isclass(obj) or what != "function":
        return
    if not callable(obj):
        return

    plots_dir = Path(app.srcdir) / "api" / "plots"

    # this only appears to be a sphinx directive, but the parsing here is very simple.
    plot_token = ".. optics-plot::"
    const_token = ".. optics-const::"

    i = 0
    plot_idx = 0
    orig_lines = lines.copy()
    for line in orig_lines:
        if line.startswith(plot_token):
            plots_dir.mkdir(exist_ok=True, parents=True)
            safe_name = "".join(c for c in name if c.isalnum() or c in (".", "-", "_"))
            safe_name += f"_{plot_idx}"

            opt_string = line[len(plot_token) :]
            opts = {}
            if opt_string != "":
                opts = ast.literal_eval(opt_string)
            assert isinstance(opts, dict)

            # replace the custom 'directive' with an actually supported reST directive.
            replace = do_plot(obj, plots_dir, safe_name, opts)
            lines[i : i + 1] = replace
            i += len(replace)
            plot_idx += 1
        elif line.startswith(const_token):
            # replace the custom 'directive' with actually supported reST content.
            replace = do_const(obj)
            lines[i : i + 1] = replace
            i += len(replace)
        else:
            i += 1


def setup(app: Sphinx) -> dict[str, bool]:
    """Register this sphinx extension."""
    app.connect("autodoc-process-docstring", process_docstring)
    return {"parallel_read_safe": True}
